@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2011, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,10 +31,12 @@
 
 package org.hsqldb;
 
-import org.hsqldb.lib.StringConverter;
-import org.hsqldb.rights.Grantee;
+import java.util.concurrent.atomic.AtomicLong;
+
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
+import org.hsqldb.lib.StringConverter;
+import org.hsqldb.rights.Grantee;
 
 /**
  * Provides Name Management for SQL objects. <p>
@@ -59,16 +61,15 @@ import org.hsqldb.error.ErrorCode;
  * than all the existing names.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.2.9
+ * @version 2.5.1
  * @since 1.7.2
  */
 public final class HsqlNameManager {
 
-    public static final String           DEFAULT_CATALOG_NAME = "PUBLIC";
     private static final HsqlNameManager staticManager = new HsqlNameManager();
 
     static {
-        staticManager.serialNumber = Integer.MIN_VALUE;
+        staticManager.serialNumber.set(Long.MIN_VALUE);
     }
 
     private static final HsqlName[] autoColumnNames       = new HsqlName[32];
@@ -82,8 +83,8 @@ public final class HsqlNameManager {
         }
     }
 
-    private int      serialNumber = 1;        // 0 is reserved in lookups
-    private int      sysNumber    = 10000;    // avoid name clash in older scripts
+    private AtomicLong serialNumber = new AtomicLong(1);     // 0 is reserved in lookups
+    private AtomicLong sysNumber = new AtomicLong(10000);    // avoid name clash in older scripts
     private HsqlName catalogName;
     private boolean  sqlRegularNames;
     HsqlName         subqueryTableName;
@@ -94,7 +95,7 @@ public final class HsqlNameManager {
 
     public HsqlNameManager(Database database) {
 
-        catalogName = new HsqlName(this, DEFAULT_CATALOG_NAME,
+        catalogName = new HsqlName(this, SqlInvariants.DEFAULT_CATALOG_NAME,
                                    SchemaObject.CATALOG, false);
         sqlRegularNames = database.sqlRegularNames;
         subqueryTableName = new HsqlName(this, SqlInvariants.SYSTEM_SUBQUERY,
@@ -210,17 +211,41 @@ public final class HsqlNameManager {
     public HsqlName newAutoName(String prefix, HsqlName schema,
                                 HsqlName parent, int type) {
 
-        HsqlName name = newAutoName(prefix, (String) null, schema, parent,
-                                    type);
+        HsqlName name = newAutoName(prefix, null, schema, parent, type);
 
         return name;
     }
 
+    public HsqlName newConstraintIndexName(HsqlName tableName,
+                                           HsqlName constName,
+                                           boolean useConstraint) {
+
+        if (constName == null) {
+            useConstraint = false;
+        }
+
+        if (useConstraint) {
+            HsqlName name = new HsqlName(this, constName.name,
+                                         SchemaObject.INDEX, false);
+
+            name.schema = tableName.schema;
+            name.parent = tableName;
+
+            return name;
+        } else {
+            String constNameString = constName == null ? null
+                                                       : constName.name;
+
+            return newAutoName("IDX", constNameString, tableName.schema,
+                               tableName, SchemaObject.INDEX);
+        }
+    }
+
     public HsqlName newSpecificRoutineName(HsqlName name) {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
-        sb.append(name.name).append('_').append(++sysNumber);
+        sb.append(name.name).append('_').append(sysNumber.incrementAndGet());
 
         HsqlName hsqlName = new HsqlName(this, sb.toString(),
                                          SchemaObject.SPECIFIC_ROUTINE,
@@ -241,19 +266,8 @@ public final class HsqlNameManager {
             return autoColumnNames[i];
         }
 
-        return new HsqlName(staticManager, "C_" + (i + 1), 0, false);
-    }
-
-    /**
-     * Column index i is 0 based, returns 1 based numbered column.
-     */
-    public static String getAutoColumnNameString(int i) {
-
-        if (i < autoColumnNames.length) {
-            return autoColumnNames[i].name;
-        }
-
-        return "C" + (i + 1);
+        return new HsqlName(staticManager, "C" + (i + 1),
+                            SchemaObject.COLUMN, false);
     }
 
     public static String getAutoNoNameColumnString(int i) {
@@ -267,7 +281,7 @@ public final class HsqlNameManager {
 
     public static String getAutoSavepointNameString(long i, int j) {
 
-        StringBuffer sb = new StringBuffer("S");
+        StringBuilder sb = new StringBuilder("S");
 
         sb.append(i).append('_').append(j);
 
@@ -280,10 +294,10 @@ public final class HsqlNameManager {
     public HsqlName newAutoName(String prefix, String namepart,
                                 HsqlName schema, HsqlName parent, int type) {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         if (prefix != null) {
-            if (prefix.length() != 0) {
+            if (!prefix.isEmpty()) {
                 sb.append("SYS_");
                 sb.append(prefix);
                 sb.append('_');
@@ -293,7 +307,7 @@ public final class HsqlNameManager {
                     sb.append('_');
                 }
 
-                sb.append(++sysNumber);
+                sb.append(sysNumber.incrementAndGet());
             }
         } else {
             sb.append(namepart);
@@ -307,11 +321,6 @@ public final class HsqlNameManager {
         return name;
     }
 
-    void resetNumbering() {
-        sysNumber    = 0;
-        serialNumber = 0;
-    }
-
     public static SimpleName getSimpleName(String name, boolean isNameQuoted) {
         return new SimpleName(name, isNameQuoted);
     }
@@ -324,7 +333,7 @@ public final class HsqlNameManager {
         private SimpleName() {}
 
         private SimpleName(String name, boolean isNameQuoted) {
-            this.name         = new String(name);
+            this.name         = name;
             this.isNameQuoted = isNameQuoted;
         }
 
@@ -335,8 +344,7 @@ public final class HsqlNameManager {
         public boolean equals(Object other) {
 
             if (other instanceof SimpleName) {
-                return ((SimpleName) other).isNameQuoted == isNameQuoted
-                       && ((SimpleName) other).name.equals(name);
+                return ((SimpleName) other).name.equals(name);
             }
 
             return false;
@@ -359,20 +367,22 @@ public final class HsqlNameManager {
         static HsqlName[] emptyArray = new HsqlName[]{};
 
         //
-        HsqlNameManager   manager;
-        public String     statementName;
-        public String     comment;
-        public HsqlName   schema;
-        public HsqlName   parent;
-        public Grantee    owner;
-        public final int  type;
-        private final int hashCode;
+        HsqlNameManager  manager;
+        public String    statementName;
+        public String    comment;
+        public HsqlName  schema;
+        public HsqlName  parent;
+        public Grantee   owner;
+        public final int type;
+
+        //
+        private final long hashCode;
 
         private HsqlName(HsqlNameManager man, int type) {
 
             manager   = man;
             this.type = type;
-            hashCode  = manager.serialNumber++;
+            hashCode  = manager.serialNumber.getAndIncrement();
         }
 
         private HsqlName(HsqlNameManager man, String name, boolean isquoted,
@@ -411,14 +421,15 @@ public final class HsqlNameManager {
                 case SchemaObject.VARIABLE : {
                     return statementName;
                 }
-                case SchemaObject.COLUMN : {
+                case SchemaObject.COLUMN :
+                case SchemaObject.PERIOD : {
                     if (parent == null
                             || SqlInvariants.SYSTEM_SUBQUERY.equals(
                                 parent.name)) {
                         return statementName;
                     }
 
-                    StringBuffer sb = new StringBuffer();
+                    StringBuilder sb = new StringBuilder();
 
                     if (schema != null) {
                         sb.append(schema.getStatementName());
@@ -433,18 +444,33 @@ public final class HsqlNameManager {
                 }
             }
 
-            if (schema == null) {
+            if (schema == null
+                    || SqlInvariants.SYSTEM_SCHEMA.equals(schema.name)) {
                 return statementName;
             }
 
-            StringBuffer sb = new StringBuffer();
+            StringBuilder sb = new StringBuilder();
 
-            if (schema != null) {
-                sb.append(schema.getStatementName());
-                sb.append('.');
+            sb.append(schema.getStatementName());
+            sb.append('.');
+            sb.append(statementName);
+
+            return sb.toString();
+        }
+
+        String getCommentSQL(String typeName) {
+
+            if (comment == null) {
+                return null;
             }
 
-            sb.append(statementName);
+            StringBuilder sb = new StringBuilder();
+
+            sb.append(Tokens.T_COMMENT).append(' ').append(Tokens.T_ON);
+            sb.append(' ').append(typeName).append(' ');
+            sb.append(getSchemaQualifiedStatementName());
+            sb.append(' ').append(Tokens.T_IS).append(' ');
+            sb.append(StringConverter.toQuotedString(comment, '\'', true));
 
             return sb.toString();
         }
@@ -460,7 +486,7 @@ public final class HsqlNameManager {
             }
 
             // get rid of the excess
-            this.name          = new String(name);
+            this.name          = name;
             this.statementName = this.name;
             this.isNameQuoted  = isquoted;
 
@@ -475,8 +501,8 @@ public final class HsqlNameManager {
                 try {
                     int temp = Integer.parseInt(name.substring(length));
 
-                    if (temp > manager.sysNumber) {
-                        manager.sysNumber = temp;
+                    if (temp > manager.sysNumber.get()) {
+                        manager.sysNumber.set(temp);
                     }
                 } catch (NumberFormatException e) {}
             }
@@ -484,7 +510,7 @@ public final class HsqlNameManager {
 
         void rename(String prefix, String name, boolean isquoted) {
 
-            StringBuffer sbname = new StringBuffer(prefix);
+            StringBuilder sbname = new StringBuilder(prefix);
 
             sbname.append('_');
             sbname.append(name);
@@ -502,16 +528,18 @@ public final class HsqlNameManager {
 
             if (other instanceof HsqlName) {
                 return hashCode == ((HsqlName) other).hashCode;
+            } else if (other instanceof SimpleName) {
+                return super.equals(other);
             }
 
             return false;
         }
 
         /**
-         * hash code for this object is its unique serial number.
+         * hash code for this object is based on its unique serial number.
          */
         public int hashCode() {
-            return hashCode;
+            return (int) (hashCode ^ (hashCode >>> 32));
         }
 
         /**
@@ -554,10 +582,6 @@ public final class HsqlNameManager {
                    + ", isNameQuoted=" + isNameQuoted + "]";
         }
 
-        public int compareTo(Object o) {
-            return hashCode - o.hashCode();
-        }
-
         /**
          * Returns true if the identifier consists of all uppercase letters
          * digits and underscore, beginning with a letter and is not in the
