@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2011, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,7 @@ import org.hsqldb.types.Type;
  * Implementation of Statement for updating result rows.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.2.7
+ * @version 2.3.2
  * @since 1.9.0
  */
 public class StatementResultUpdate extends StatementDML {
@@ -79,7 +79,7 @@ public class StatementResultUpdate extends StatementDML {
         } catch (Throwable t) {
             clearStructures(session);
 
-            result = Result.newErrorResult(t, null);
+            result = Result.newErrorResult(t);
         }
 
         return result;
@@ -98,7 +98,7 @@ public class StatementResultUpdate extends StatementDML {
             case ResultConstants.UPDATE_CURSOR : {
                 row = getRow(session, args);
 
-                /**
+                /*
                  * @todo - in 2PL mode isDeleted() always returns false.
                  * While write lock prevents delete by other transactions,
                  * same-transaction deletes are not caught
@@ -143,14 +143,14 @@ public class StatementResultUpdate extends StatementDML {
 
                 list.addRow(row);
                 list.endMainDataSet();
-                delete(session, baseTable, list);
+                delete(session, baseTable, list, null);
 
                 break;
             }
             case ResultConstants.INSERT_CURSOR : {
                 Object[] data = baseTable.getNewRowData(session);
 
-                for (int i = 0; i < data.length; i++) {
+                for (int i = 0; i < baseColumnMap.length; i++) {
                     data[baseColumnMap[i]] = args[i];
                 }
 
@@ -187,15 +187,23 @@ public class StatementResultUpdate extends StatementDML {
         return row;
     }
 
-    void setRowActionProperties(Result result, int action, Table table,
-                                Type[] types, int[] columnMap) {
+    void setRowActionProperties(Result result, int action,
+                                StatementQuery statement, Type[] types) {
 
-        this.result             = result;
-        this.actionType         = action;
-        this.baseTable          = table;
-        this.types              = types;
-        this.baseColumnMap      = columnMap;
-        this.writeTableNames[0] = table.getName();
+        QueryExpression qe = statement.queryExpression;
+
+        this.result               = result;
+        this.actionType           = action;
+        this.baseTable            = qe.getBaseTable();
+        this.types                = types;
+        this.baseColumnMap        = qe.getBaseTableColumnMap();
+        this.writeTableNames[0]   = baseTable.getName();
+        this.rangeVariables       = qe.getRangeVariables();
+        this.targetRangeVariables = qe.getRangeVariables();
+
+        // used for statement logging - needs improvements to list only the updated values
+        this.sql               = statement.getSQL();
+        this.parameterMetaData = qe.getMetaData();
     }
 
     void checkAccessRights(Session session) {
