@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,32 +31,33 @@
 
 package org.hsqldb.lib;
 
+import java.util.Arrays;
 import java.util.NoSuchElementException;
 
 /**
- * Maintains an ordered  integer->integer lookup table, consisting of two
- * columns, one for keys, the other for values.
+ * Maintains an ordered  {@code integer->integer} lookup table, consisting of two
+ * columns, one for keys, the other for values. Equal keys are allowed.
  *
  * The table is sorted on either the key or value column, depending on the calls to
  * setKeysSearchTarget() or setValuesSearchTarget(). By default, the table is
- * sorted on values.<p>
+ * sorted on values. Equal values are sorted by key.<p>
  *
  * findXXX() methods return the array index into the list
  * pair containing a matching key or value, or  or -1 if not found.<p>
  *
- * Sorting methods originally contributed by Tony Lai.
+ * Sorting methods originally contributed by Tony Lai (tony_lai@users dot sourceforge.net).
+ * Non-recursive implementation of fast quicksort added by Sergio Bossa sbtourist@users dot sourceforge.net)
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.8.0
+ * @version 2.5.1
  * @since 1.8.0
  */
-public class DoubleIntIndex implements IntLookup {
+public class DoubleIntIndex implements LongLookup {
 
     private int           count = 0;
     private int           capacity;
     private boolean       sorted       = true;
-    private boolean       sortOnValues = true;
-    private boolean       hasChanged;
+    private boolean       sortOnValues = false;
     private final boolean fixedSize;
     private int[]         keys;
     private int[]         values;
@@ -64,16 +65,19 @@ public class DoubleIntIndex implements IntLookup {
 //
     private int targetSearchValue;
 
+    public DoubleIntIndex(int capacity) {
+        this(capacity, false);
+    }
+
     public DoubleIntIndex(int capacity, boolean fixedSize) {
 
         this.capacity  = capacity;
         keys           = new int[capacity];
         values         = new int[capacity];
         this.fixedSize = fixedSize;
-        hasChanged     = true;
     }
 
-    public synchronized int getKey(int i) {
+    public int getKey(int i) {
 
         if (i < 0 || i >= count) {
             throw new IndexOutOfBoundsException();
@@ -82,7 +86,20 @@ public class DoubleIntIndex implements IntLookup {
         return keys[i];
     }
 
-    public synchronized int getValue(int i) {
+    public long getLongKey(int i) {
+
+        if (i < 0 || i >= count) {
+            throw new IndexOutOfBoundsException();
+        }
+
+        return keys[i] & 0xffffffffL;
+    }
+
+    public long getLongValue(int i) {
+        return values[i];
+    }
+
+    public int getValue(int i) {
 
         if (i < 0 || i >= count) {
             throw new IndexOutOfBoundsException();
@@ -92,11 +109,11 @@ public class DoubleIntIndex implements IntLookup {
     }
 
     /**
-     * Modifies an existing pair.
+     * Modifies an existing pair at the given index.
      * @param i the index
-     * @param key the key
+     * @param key the new key
      */
-    public synchronized void setKey(int i, int key) {
+    public void setKey(int i, int key) {
 
         if (i < 0 || i >= count) {
             throw new IndexOutOfBoundsException();
@@ -110,11 +127,11 @@ public class DoubleIntIndex implements IntLookup {
     }
 
     /**
-     * Modifies an existing pair.
+     * Modifies an existing pair at the given index.
      * @param i the index
-     * @param value the value
+     * @param value the new value
      */
-    public synchronized void setValue(int i, int value) {
+    public void setValue(int i, int value) {
 
         if (i < 0 || i >= count) {
             throw new IndexOutOfBoundsException();
@@ -127,14 +144,68 @@ public class DoubleIntIndex implements IntLookup {
         values[i] = value;
     }
 
-    public synchronized int size() {
+    /**
+     * Modifies an existing pair at the given index.
+     * @param i the index
+     * @param value the new value
+     */
+    public void setLongValue(int i, long value) {
+
+        if (i < 0 || i >= count) {
+            throw new IndexOutOfBoundsException();
+        }
+
+        if (sortOnValues) {
+            sorted = false;
+        }
+
+        values[i] = (int) value;
+    }
+
+    public int size() {
         return count;
     }
 
-    public synchronized int capacity() {
+    public int capacity() {
         return capacity;
     }
 
+    public int[] getKeys() {
+        return keys;
+    }
+
+    public int[] getValues() {
+        return values;
+    }
+
+    public long getTotalValues() {
+
+        long total = 0;
+
+        for (int i = 0; i < count; i++) {
+            total += values[i];
+        }
+
+        return total;
+    }
+
+    public void setSize(int newSize) {
+        count = newSize;
+    }
+
+    public boolean addUnsorted(long key, long value) {
+
+        if (key > Integer.MAX_VALUE || key < Integer.MIN_VALUE) {
+            throw new IllegalArgumentException();
+        }
+
+        if (value > Integer.MAX_VALUE || value < Integer.MIN_VALUE) {
+            throw new IllegalArgumentException();
+        }
+
+        return addUnsorted((int) key, (int) value);
+    }
+
     /**
      * Adds a pair into the table.
      *
@@ -142,7 +213,7 @@ public class DoubleIntIndex implements IntLookup {
      * @param value the value
      * @return true or false depending on success
      */
-    public synchronized boolean addUnsorted(int key, int value) {
+    public boolean addUnsorted(int key, int value) {
 
         if (count == capacity) {
             if (fixedSize) {
@@ -164,7 +235,6 @@ public class DoubleIntIndex implements IntLookup {
             }
         }
 
-        hasChanged    = true;
         keys[count]   = key;
         values[count] = value;
 
@@ -173,6 +243,39 @@ public class DoubleIntIndex implements IntLookup {
         return true;
     }
 
+    public boolean addUnsorted(LongLookup other) {
+
+        if (!ensureCapacityToAdd(other.size())) {
+            return false;
+        }
+
+        sorted = false;
+
+        for (int i = 0; i < other.size(); i++) {
+            long key   = other.getLongKey(i);
+            long value = other.getLongValue(i);
+
+            this.addUnsorted(key, value);
+        }
+
+        return true;
+    }
+
+    private boolean ensureCapacityToAdd(int extra) {
+
+        if (count + extra > capacity) {
+            if (fixedSize) {
+                return false;
+            } else {
+                while (count + extra > capacity) {
+                    doubleCapacity();
+                }
+            }
+        }
+
+        return true;
+    }
+
     /**
      * Adds a key, value pair into the table with the guarantee that the key
      * is equal or larger than the largest existing key. This prevents a sort
@@ -182,7 +285,7 @@ public class DoubleIntIndex implements IntLookup {
      * @param value the value
      * @return true or false depending on success
      */
-    public synchronized boolean addSorted(int key, int value) {
+    public boolean addSorted(int key, int value) {
 
         if (count == capacity) {
             if (fixedSize) {
@@ -196,6 +299,9 @@ public class DoubleIntIndex implements IntLookup {
             if (sortOnValues) {
                 if (value < values[count - 1]) {
                     return false;
+                } else if (value == values[count - 1]
+                           && key < keys[count - 1]) {
+                    return false;
                 }
             } else {
                 if (key < keys[count - 1]) {
@@ -204,7 +310,6 @@ public class DoubleIntIndex implements IntLookup {
             }
         }
 
-        hasChanged    = true;
         keys[count]   = key;
         values[count] = value;
 
@@ -220,7 +325,11 @@ public class DoubleIntIndex implements IntLookup {
      * @param value the value
      * @return true or false depending on success
      */
-    public synchronized boolean addUnique(int key, int value) {
+    public boolean addUnique(int key, int value) {
+
+        if (sortOnValues) {
+            throw new IllegalArgumentException();
+        }
 
         if (count == capacity) {
             if (fixedSize) {
@@ -243,8 +352,6 @@ public class DoubleIntIndex implements IntLookup {
             return false;
         }
 
-        hasChanged = true;
-
         if (count != i) {
             moveRows(i, i + 1, count - i);
         }
@@ -258,13 +365,136 @@ public class DoubleIntIndex implements IntLookup {
     }
 
     /**
-     * Adds a pair, maintaining sorted order
+     * Removes the (unique) key and its value. Must be sorted on key.
+     *
+     * @param key the key to remove
+     * @return true or false depending on success
+     */
+    public boolean removeKey(int key) {
+
+        if (sortOnValues) {
+            throw new IllegalArgumentException();
+        }
+
+        if (!sorted) {
+            fastQuickSort();
+        }
+
+        targetSearchValue = key;
+
+        int i = binarySlotSearch(false);
+
+        if (i == count) {
+            return false;
+        }
+
+        if (keys[i] != key) {
+            return false;
+        }
+
+        remove(i);
+
+        return true;
+    }
+
+    /**
+     * Updates the value if key is present, or adds the key/value paire.
+     * Must be sorted on key.
+     *
+     * @param key the key to add or find
+     * @param value the value to add or update
+     * @return true or false depending on success
+     */
+    public boolean addOrReplaceUnique(int key, int value) {
+
+        if (sortOnValues) {
+            throw new IllegalArgumentException();
+        }
+
+        if (!sorted) {
+            fastQuickSort();
+        }
+
+        targetSearchValue = key;
+
+        int i = binarySlotSearch(false);
+
+        if (i < count) {
+            if (keys[i] == key) {
+                values[i] = value;
+
+                return true;
+            }
+
+            if (count == capacity) {
+                if (fixedSize) {
+                    return false;
+                } else {
+                    doubleCapacity();
+                }
+            }
+
+            moveRows(i, i + 1, count - i);
+        }
+
+        keys[i]   = key;
+        values[i] = value;
+
+        count++;
+
+        return true;
+    }
+
+    /**
+     * Used for values as counters. Adds the value to the existing value for the
+     * key. Or adds the key - value pair.
+     *
+     * @param key the key to update or add
+     * @param value the count to add
+     * @return the new count for the key
+     */
+    public int addCount(int key, int value) {
+
+        sortOnValues = false;
+
+        if (addUnique(key, value)) {
+            return value;
+        }
+
+        targetSearchValue = key;
+
+        int i = this.binarySlotSearch(false);
+
+        values[i] += value;
+
+        return values[i];
+    }
+
+    public int addCount(int key) {
+        return addCount(key, 1);
+    }
+
+    public int add(long key, long value) {
+
+        if (key > Integer.MAX_VALUE || key < Integer.MIN_VALUE) {
+            throw new IllegalArgumentException();
+        }
+
+        if (value > Integer.MAX_VALUE || value < Integer.MIN_VALUE) {
+            throw new IllegalArgumentException();
+        }
+
+        return add((int) key, (int) value);
+    }
+
+    /**
+     * Adds a pair, maintaining sort order on
      * current search target column.
      * @param key the key
      * @param value the value
-     * @return true or false depending on success
+     * @return index of added key or -1 if full
      */
-    public synchronized int add(int key, int value) {
+    public int add(int key, int value) {
 
         if (count == capacity) {
             if (fixedSize) {
@@ -281,13 +511,7 @@ public class DoubleIntIndex implements IntLookup {
         targetSearchValue = sortOnValues ? value
                                          : key;
 
-        int i = binarySlotSearch();
-
-        if (i == -1) {
-            return i;
-        }
-
-        hasChanged = true;
+        int i = binarySlotSearch(true);
 
         if (count != i) {
             moveRows(i, i + 1, count - i);
@@ -301,7 +525,16 @@ public class DoubleIntIndex implements IntLookup {
         return i;
     }
 
-    public int lookupFirstEqual(int key) throws NoSuchElementException {
+    public long lookup(long key) throws NoSuchElementException {
+
+        if (key > Integer.MAX_VALUE || key < Integer.MIN_VALUE) {
+            throw new NoSuchElementException();
+        }
+
+        return lookup((int) key);
+    }
+
+    public int lookup(int key) throws NoSuchElementException {
 
         if (sortOnValues) {
             sorted       = false;
@@ -317,6 +550,55 @@ public class DoubleIntIndex implements IntLookup {
         return getValue(i);
     }
 
+    public long lookup(long key, long def) {
+
+        if (key > Integer.MAX_VALUE || key < Integer.MIN_VALUE) {
+            return def;
+        }
+
+        if (sortOnValues) {
+            sorted       = false;
+            sortOnValues = false;
+        }
+
+        int i = findFirstEqualKeyIndex((int) key);
+
+        if (i == -1) {
+            return def;
+        }
+
+        return getValue(i);
+    }
+
+    public int lookup(int key, int def) {
+
+        if (sortOnValues) {
+            sorted       = false;
+            sortOnValues = false;
+        }
+
+        int i = findFirstEqualKeyIndex(key);
+
+        if (i == -1) {
+            return def;
+        }
+
+        return getValue(i);
+    }
+
+    public void clear() {
+        removeAll();
+    }
+
+    public LongLookup duplicate() {
+
+        DoubleIntIndex duplicate = new DoubleIntIndex(capacity);
+
+        copyTo(duplicate);
+
+        return duplicate;
+    }
+
     public int lookupFirstGreaterEqual(int key) throws NoSuchElementException {
 
         if (sortOnValues) {
@@ -333,7 +615,7 @@ public class DoubleIntIndex implements IntLookup {
         return getValue(i);
     }
 
-    public synchronized void setValuesSearchTarget() {
+    public void setValuesSearchTarget() {
 
         if (!sortOnValues) {
             sorted = false;
@@ -342,7 +624,7 @@ public class DoubleIntIndex implements IntLookup {
         sortOnValues = true;
     }
 
-    public synchronized void setKeysSearchTarget() {
+    public void setKeysSearchTarget() {
 
         if (sortOnValues) {
             sorted = false;
@@ -355,7 +637,7 @@ public class DoubleIntIndex implements IntLookup {
      * @param value the value
      * @return the index
      */
-    public synchronized int findFirstGreaterEqualKeyIndex(int value) {
+    public int findFirstGreaterEqualKeyIndex(int value) {
 
         int index = findFirstGreaterEqualSlotIndex(value);
 
@@ -367,7 +649,7 @@ public class DoubleIntIndex implements IntLookup {
      * @param value the value
      * @return the index
      */
-    public synchronized int findFirstEqualKeyIndex(int value) {
+    public int findFirstEqualKeyIndex(int value) {
 
         if (!sorted) {
             fastQuickSort();
@@ -378,6 +660,47 @@ public class DoubleIntIndex implements IntLookup {
         return binaryFirstSearch();
     }
 
+    public boolean compactLookupAsIntervals() {
+
+        if (size() == 0) {
+            return false;
+        }
+
+        setKeysSearchTarget();
+
+        if (!sorted) {
+            fastQuickSort();
+        }
+
+        int base = 0;
+
+        for (int i = 1; i < count; i++) {
+            long limit = keys[base] + values[base];
+
+            if (limit == keys[i]) {
+                values[base] += values[i];    // base updated
+            } else {
+                base++;
+
+                keys[base]   = keys[i];
+                values[base] = values[i];
+            }
+        }
+
+        for (int i = base + 1; i < count; i++) {
+            keys[i]   = 0;
+            values[i] = 0;
+        }
+
+        if (count != base + 1) {
+            setSize(base + 1);
+
+            return true;
+        }
+
+        return false;
+    }
+
     /**
      * This method is similar to findFirstGreaterEqualKeyIndex(int) but
      * returns the index of the empty row past the end of the array if
@@ -386,7 +709,7 @@ public class DoubleIntIndex implements IntLookup {
      * @param value the value
      * @return the index
      */
-    public synchronized int findFirstGreaterEqualSlotIndex(int value) {
+    public int findFirstGreaterEqualSlotIndex(int value) {
 
         if (!sorted) {
             fastQuickSort();
@@ -394,7 +717,7 @@ public class DoubleIntIndex implements IntLookup {
 
         targetSearchValue = value;
 
-        return binarySlotSearch();
+        return binarySlotSearch(false);
     }
 
     /**
@@ -411,7 +734,7 @@ public class DoubleIntIndex implements IntLookup {
         int found   = count;
 
         while (low < high) {
-            mid     = (low + high) / 2;
+            mid     = (low + high) >>> 1;
             compare = compare(mid);
 
             if (compare < 0) {
@@ -429,10 +752,13 @@ public class DoubleIntIndex implements IntLookup {
     }
 
     /**
-     * Returns the index of the lowest element > the given search target
-     *     @return the index
+     * Returns the index of the lowest element >= the given search target, or
+     * count.
+     *
+     * @return the index or count.
+     * @param fullCompare ignored
      */
-    private int binaryGreaterSearch() {
+    private int binarySlotSearch(boolean fullCompare) {
 
         int low     = 0;
         int high    = count;
@@ -440,34 +766,7 @@ public class DoubleIntIndex implements IntLookup {
         int compare = 0;
 
         while (low < high) {
-            mid     = (low + high) / 2;
-            compare = compare(mid);
-
-            if (compare < 0) {
-                high = mid;
-            } else {
-                low = mid + 1;
-            }
-        }
-
-        return low == count ? -1
-                            : low;
-    }
-
-    /**
-     * Returns the index of the lowest element >= the given search target,
-     * or count
-     *     @return the index
-     */
-    private int binarySlotSearch() {
-
-        int low     = 0;
-        int high    = count;
-        int mid     = 0;
-        int compare = 0;
-
-        while (low < high) {
-            mid     = (low + high) / 2;
+            mid     = (low + high) >>> 1;
             compare = compare(mid);
 
             if (compare <= 0) {
@@ -493,7 +792,7 @@ public class DoubleIntIndex implements IntLookup {
         int compare = 0;
 
         while (low < high) {
-            mid     = (low + high) / 2;
+            mid     = (low + high) >>> 1;
             compare = compare(mid);
 
             if (compare < 0) {
@@ -508,7 +807,102 @@ public class DoubleIntIndex implements IntLookup {
         return low;
     }
 
-    private synchronized void fastQuickSort() {
+    public void sortOnKeys() {
+
+        sortOnValues = false;
+
+        fastQuickSort();
+    }
+
+    public void sortOnValues() {
+
+        sortOnValues = true;
+
+        fastQuickSort();
+    }
+
+    public void sort() {
+
+        if (sortOnValues || count <= 1024 * 16) {
+            fastQuickSortRecursive();
+        } else {
+            fastQuickSort();
+        }
+    }
+
+    /**
+     * fast quicksort using a stack on the heap to reduce stack use
+     */
+    private void fastQuickSort() {
+
+        DoubleIntIndex indices   = new DoubleIntIndex(32768);
+        int            threshold = 16;
+
+        indices.push(0, count - 1);
+
+        while (indices.size() > 0) {
+            int start = indices.peekKey();
+            int end   = indices.peekValue();
+
+            indices.pop();
+
+            if (end - start >= threshold) {
+                int pivot = partition(start, end);
+
+                indices.push(start, pivot - 1);
+                indices.push(pivot + 1, end);
+            }
+        }
+
+        insertionSort(0, count - 1);
+
+        sorted = true;
+    }
+
+    private int partition(int start, int end) {
+
+        int pivot = (start + end) >>> 1;
+
+        // pivot is median of three values
+        if (keys[pivot] < keys[(start + pivot) >>> 1]) {
+            swap(pivot, (start + pivot) >>> 1);
+        }
+
+        if (keys[(end + pivot) >>> 1] < keys[(start + pivot) >>> 1]) {
+            swap((end + pivot) >>> 1, (start + pivot) >>> 1);
+        }
+
+        if (keys[(end + pivot) >>> 1] < keys[pivot]) {
+            swap((end + pivot) >>> 1, pivot);
+        }
+
+        int pivotValue = keys[pivot];
+        int i          = start - 1;
+        int j          = end;
+
+        swap(pivot, end);
+
+        for (;;) {
+            while (keys[++i] < pivotValue) {}
+
+            while (pivotValue < keys[--j]) {}
+
+            if (j < i) {
+                break;
+            }
+
+            swap(i, j);
+        }
+
+        swap(i, end);
+
+        return i;
+    }
+
+    /**
+     * fast quicksort with recursive quicksort implementation
+     */
+    private void fastQuickSortRecursive() {
 
         quickSort(0, count - 1);
         insertionSort(0, count - 1);
@@ -524,10 +918,10 @@ public class DoubleIntIndex implements IntLookup {
         int v;
 
         if ((r - l) > M) {
-            i = (r + l) / 2;
+            i = (r + l) >>> 1;
 
             if (lessThan(i, l)) {
-                swap(l, i);    // Tri-Median Methode!
+                swap(l, i);    // Tri-Median Method!
             }
 
             if (lessThan(r, l)) {
@@ -616,13 +1010,15 @@ public class DoubleIntIndex implements IntLookup {
                 return 1;
             } else if (targetSearchValue < values[i]) {
                 return -1;
+            } else {
+                return 0;
             }
-        } else {
-            if (targetSearchValue > keys[i]) {
-                return 1;
-            } else if (targetSearchValue < keys[i]) {
-                return -1;
-            }
+        }
+
+        if (targetSearchValue > keys[i]) {
+            return 1;
+        } else if (targetSearchValue < keys[i]) {
+            return -1;
         }
 
         return 0;
@@ -639,11 +1035,13 @@ public class DoubleIntIndex implements IntLookup {
         if (sortOnValues) {
             if (values[i] < values[j]) {
                 return true;
+            } else if (values[i] > values[j]) {
+                return false;
             }
-        } else {
-            if (keys[i] < keys[j]) {
-                return true;
-            }
+        }
+
+        if (keys[i] < keys[j]) {
+            return true;
         }
 
         return false;
@@ -663,24 +1061,32 @@ public class DoubleIntIndex implements IntLookup {
 
     public void removeRange(int start, int limit) {
 
-        moveRows(limit, start, count - limit);
+        ArrayUtil.adjustArray(ArrayUtil.CLASS_CODE_INT, keys, count, start,
+                              start - limit);
+        ArrayUtil.adjustArray(ArrayUtil.CLASS_CODE_INT, values, count, start,
+                              start - limit);
 
         count -= (limit - start);
     }
 
     public void removeAll() {
 
-        hasChanged = true;
+        Arrays.fill(keys, 0);
+        Arrays.fill(values, 0);
 
-        ArrayUtil.clearArray(ArrayUtil.CLASS_CODE_INT, keys, 0, count);
-        ArrayUtil.clearArray(ArrayUtil.CLASS_CODE_INT, values, 0, count);
-
-        count = 0;
+        count  = 0;
+        sorted = true;
     }
 
-    public final synchronized void remove(int position) {
+    public void copyTo(DoubleIntIndex other) {
 
-        hasChanged = true;
+        System.arraycopy(keys, 0, other.keys, 0, count);
+        System.arraycopy(values, 0, other.values, 0, count);
+        other.setSize(count);
+        other.sorted = false;
+    }
+
+    public final void remove(int position) {
 
         moveRows(position + 1, position, count - position - 1);
 
@@ -689,4 +1095,46 @@ public class DoubleIntIndex implements IntLookup {
         keys[count]   = 0;
         values[count] = 0;
     }
+
+    /**
+     * peek the key at top of stack. Uses the data structure as a stack.
+     * @return int key
+     */
+    int peekKey() {
+        return getKey(count - 1);
+    }
+
+    /**
+     * peek the value at top of stack
+     * @return int value
+     */
+    int peekValue() {
+        return getValue(count - 1);
+    }
+
+    /**
+     * pop the pair at top of stack
+     * @return boolean if there was an element
+     */
+    boolean pop() {
+
+        if (count > 0) {
+            count--;
+
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * push key - value pair
+     *
+     * @return boolean true if successful
+     * @param key the key
+     * @param value the value
+     */
+    boolean push(int key, int value) {
+        return addUnsorted(key, value);
+    }
 }
