@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,6 +37,8 @@ import java.sql.SQLException;
 
 import org.hsqldb.lib.FileUtil;
 
+import java.sql.Statement;
+
 /** test various text table features
  *
  * @author frank.schoenheit@sun.com
@@ -109,8 +111,10 @@ public class TestTextTable extends TestBase {
             try {
                 String completeFileName = m_name + ".csv";
 
+                FileUtil.getFileUtil().delete(completeFileName);
+
                 textFile = new PrintStream(
-                    FileUtil.getFileUtil().openOutputStreamElement(
+                    FileUtil.getFileUtil().openOutputStreamElementAppend(
                         completeFileName));
 
                 new java.io.File(completeFileName).deleteOnExit();
@@ -119,8 +123,8 @@ public class TestTextTable extends TestBase {
             }
 
             for (int row = 0; row < m_data.length; ++row) {
-                StringBuffer buf      = new StringBuffer();
-                int          colCount = m_data[row].length;
+                StringBuilder buf      = new StringBuilder();
+                int           colCount = m_data[row].length;
 
                 for (int col = 0; col < colCount; ++col) {
                     buf.append(m_data[row][col].toString());
@@ -143,18 +147,20 @@ public class TestTextTable extends TestBase {
         private void createTable(java.sql.Connection connection)
         throws SQLException {
 
-            String createTable = "DROP TABLE \"" + m_name + "\" IF EXISTS;";
+            String dropTable = "DROP TABLE \"" + m_name + "\" IF EXISTS;";
 
-            createTable += "CREATE TEXT TABLE \"" + m_name + "\" ( "
+            String createTable = "CREATE TEXT TABLE \"" + m_name + "\" ( "
                            + m_columnSpec + " );";
 
-            connection.createStatement().execute(createTable);
+            Statement statement = connection.createStatement();
+            statement.execute(dropTable);
+            statement.execute(createTable);
 
             boolean test = isReadOnly(m_name);
             String setTableSource = "SET TABLE \"" + m_name + "\" SOURCE \""
                                     + getDataSourceSpec() + "\"";
 
-            connection.createStatement().execute(setTableSource);
+            statement.execute(setTableSource);
         }
     }
     ;
@@ -163,9 +169,9 @@ public class TestTextTable extends TestBase {
         "ID INTEGER PRIMARY KEY, \"name\" VARCHAR(20)", "\t", "\\t",
         new Object[][] {
         new Object[] {
-            new Integer(1), "Apples"
+            Integer.valueOf(1), "Apples"
         }, new Object[] {
-            new Integer(2), "Oranges"
+            Integer.valueOf(2), "Oranges"
         }
     });
     TextTableDescriptor m_customers = new TextTableDescriptor("customers",
@@ -173,19 +179,19 @@ public class TestTextTable extends TestBase {
         + "\"address\" VARCHAR(50)," + "\"city\" VARCHAR(50),"
         + "\"postal\" VARCHAR(50)", ";", "\\semi", new Object[][] {
         new Object[] {
-            new Integer(1), "Food, Inc.", "Down Under", "Melbourne", "509"
+            Integer.valueOf(1), "Food, Inc.", "Down Under", "Melbourne", "509"
         }, new Object[] {
-            new Integer(2), "Simply Delicious", "Down Under", "Melbourne",
+            Integer.valueOf(2), "Simply Delicious", "Down Under", "Melbourne",
             "518"
         }, new Object[] {
-            new Integer(3), "Pure Health", "10 Fish St.", "San Francisco",
+            Integer.valueOf(3), "Pure Health", "10 Fish St.", "San Francisco",
             "94107"
         }
     });
 
     /** Creates a new instance of TestTextTable */
     public TestTextTable(String testName) {
-        super(testName, null, false, false);
+        super(testName, "jdbc:hsqldb:file:test");
     }
 
     /**
@@ -212,7 +218,7 @@ public class TestTextTable extends TestBase {
         }
     }
 
-    public void setUp() {
+    public void setUp() throws Exception {
 
         super.setUp();
         setupTextFiles();
@@ -413,7 +419,7 @@ public class TestTextTable extends TestBase {
                          + "\" VALUES ( 3, 'Pears' )");
         verifyTableContent(m_products.getName(),
                            m_products.appendRowData(new Object[] {
-            new Integer(3), "Pears"
+            Integer.valueOf(3), "Pears"
         }));
 
         // check whether the PK constraint works
@@ -487,7 +493,8 @@ public class TestTextTable extends TestBase {
             // create a malformed file
             String fileName = "malformed.csv";
             PrintStream textFile = new PrintStream(
-                FileUtil.getFileUtil().openOutputStreamElement(fileName));
+                FileUtil.getFileUtil().openOutputStreamElementAppend(
+                    fileName));
 
             textFile.println("not a number;some text");
             textFile.close();
@@ -503,46 +510,47 @@ public class TestTextTable extends TestBase {
             } catch (java.sql.SQLException es) {    /* that's expected here */
             }
 
-/*
             // new - a malformed data source assignment by user should not survive
             // and should revert to the existing one
             assertTrue(
-                "A table with an invalid data source should fall back to read-only.",
-                isReadOnly(m_products.getName()));
-
+                "A table with an invalid data source should fall back to original read-only.",
+                !isReadOnly(m_products.getName()));
             assertEquals(
                 "A data source which cannot be set should nonetheless be remembered.",
-                newDataSourceSpec, getDataSourceSpec(m_products.getName()));
-*/
+                m_products.getDataSourceSpec(),
+                getDataSourceSpec(m_products.getName()));
 
             // the data source spec should even survive a shutdown
             executeStatement("SHUTDOWN");
 
             m_connection = newConnection();
             m_statement  = m_connection.createStatement();
-/*
-            assertEquals(
-                "A data source pointing to a mailformed file should survive a database shutdown.",
-                newDataSourceSpec, getDataSourceSpec(m_products.getName()));
+
+            assertEquals("A data source should survive a database shutdown.",
+                         m_products.getDataSourceSpec(),
+                         getDataSourceSpec(m_products.getName()));
             assertTrue(
-                "After shutdown and DB-reconnect, the table with a malformed source should be read-only, again.",
-                isReadOnly(m_products.getName()));
-*/
+                "After shutdown and DB-reconnect, the table should keepe read-only attribute.",
+                !isReadOnly(m_products.getName()));
 
             // reconnect after fixing the file
+            FileUtil.getFileUtil().delete(fileName);
+
             textFile = new PrintStream(
-                FileUtil.getFileUtil().openOutputStreamElement(fileName));
+                FileUtil.getFileUtil().openOutputStreamElementAppend(
+                    fileName));
 
             textFile.println("1;some text");
             textFile.close();
-            executeStatement(sqlSetTable + " SOURCE ON");
+            m_statement.execute(sqlSetTable + " SOURCE \"" + newDataSourceSpec
+                                + "\"");
             assertFalse(
                 "The file was fixed, reconnect was successful, so the table shouldn't be read-only.",
                 isReadOnly(m_products.getName()));
 
             // finally re-create the proper version of the table for any further tests
-            m_products.createTextFile();
-            m_products.createTable(m_connection);
+            m_statement.execute(sqlSetTable + " SOURCE \""
+                                + m_products.getDataSourceSpec() + "\"");
             verifyTableContent(m_products.getName(), m_products.getData());
         } catch (junit.framework.AssertionFailedError e) {
             throw e;
