@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2011, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,30 +31,35 @@
 
 package org.hsqldb.types;
 
+import org.hsqldb.OpTypes;
 import org.hsqldb.Session;
 import org.hsqldb.SessionInterface;
 import org.hsqldb.Tokens;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.jdbc.JDBCClobClient;
-import org.hsqldb.lib.StringConverter;
 
 /**
  * Type object for CLOB.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.2.9
+ * @version 2.5.0
  * @since 1.9.0
  */
 public final class ClobType extends CharacterType {
 
-    public static final long maxClobPrecision = 1024L * 1024 * 1024 * 1024;
-    public static final int  defaultClobSize  = 1024 * 1024 * 16;
+    public static final long maxClobPrecision     = 1024L * 1024 * 1024 * 1024;
+    public static final int  defaultClobSize      = 1024 * 1024 * 1024;
+    public static final int  defaultShortClobSize = 16 * 1024 * 1024;
 
     public ClobType(long precision) {
         super(Types.SQL_CLOB, precision);
     }
 
+    public ClobType(Collation collation, long precision) {
+        super(collation, Types.SQL_CLOB, precision);
+    }
+
     public int displaySize() {
         return precision > Integer.MAX_VALUE ? Integer.MAX_VALUE
                                              : (int) precision;
@@ -94,7 +99,7 @@ public final class ClobType extends CharacterType {
             }
         }
 
-        StringBuffer sb = new StringBuffer(16);
+        StringBuilder sb = new StringBuilder(16);
 
         sb.append(getNameString());
         sb.append('(');
@@ -117,8 +122,12 @@ public final class ClobType extends CharacterType {
         return true;
     }
 
-    /** @todo - collation comparison */
+    /* @todo - collation comparison */
     public int compare(Session session, Object a, Object b) {
+        return compare(session, a, b, OpTypes.EQUAL);
+    }
+
+    public int compare(Session session, Object a, Object b, int opType) {
 
         if (a == b) {
             return 0;
@@ -133,8 +142,10 @@ public final class ClobType extends CharacterType {
         }
 
         if (b instanceof String) {
+            long lobId = ((ClobData) a).getId();
+
             return session.database.lobManager.compare(collation,
-                    (ClobData) a, (String) b);
+                    lobId, (String) b);
         }
 
         return session.database.lobManager.compare(collation, (ClobData) a,
@@ -168,7 +179,7 @@ public final class ClobType extends CharacterType {
             return null;
         }
 
-        return ((ClobData) a).toString();
+        return Long.toString(((ClobData) a).getId());
     }
 
     public String convertToSQLString(Object a) {
@@ -177,9 +188,7 @@ public final class ClobType extends CharacterType {
             return Tokens.T_NULL;
         }
 
-        String s = convertToString(a);
-
-        return StringConverter.toQuotedString(s, '\'', true);
+        return convertToString(a);
     }
 
     public Object convertJavaToSQL(SessionInterface session, Object a) {
